import globalPluginHandler
import gui
import wx
import core
import re
from scriptHandler import script

class ResultDialog(wx.Dialog):
    def __init__(self, resultText):
        super().__init__(None, size=(250, 150))
        panel = wx.Panel(self)
        sizer = wx.BoxSizer(wx.VERTICAL)

        resultLabel = wx.StaticText(panel, label=resultText)
        okButton = wx.Button(panel, wx.ID_OK, label="OK")
        okButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())

        sizer.Add(resultLabel, 0, wx.ALL | wx.CENTER, 20)
        sizer.Add(okButton, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        panel.SetSizer(sizer)

        accel_tbl = wx.AcceleratorTable([
            (wx.ACCEL_NORMAL, wx.WXK_ESCAPE, wx.ID_CANCEL),
            (wx.ACCEL_NORMAL, wx.WXK_RETURN, wx.ID_OK),
        ])
        self.SetAcceleratorTable(accel_tbl)

        self.Bind(wx.EVT_BUTTON, self.onAnyButton)
        self.Bind(wx.EVT_CLOSE, self.onClose)
        self.Centre()
        self.Show()

    def onAnyButton(self, event):
        self.Close()

    def onClose(self, event):
        self.Destroy()

class CalculatorDialog(wx.Frame):
    def __init__(self):
        super().__init__(None, title="Kalkulator NVDA", size=(350, 200))
        panel = wx.Panel(self)

        sizer = wx.BoxSizer(wx.VERTICAL)

        self.inputCtrl = wx.TextCtrl(panel)
        sizer.Add(wx.StaticText(panel, label="Wpisz działanie (np. 50%10 albo 25%20 + 2):"), 0, wx.ALL, 5)
        sizer.Add(self.inputCtrl, 0, wx.ALL | wx.EXPAND, 5)

        button = wx.Button(panel, label="Oblicz")
        button.Bind(wx.EVT_BUTTON, self.onCalculate)
        sizer.Add(button, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        panel.SetSizer(sizer)
        self.Centre()
        self.Show()

    def onCalculate(self, event):
        expression = self.inputCtrl.GetValue()
        try:
            safe_expr = expression.replace(" ", "")

            # 👇 Zamieniamy np. 50%10 ➜ (50/100)*10
            safe_expr = re.sub(r'(\d+)%(\d+)', r'(\1/100)*\2', safe_expr)

            if not all(c in "0123456789+-*/().," for c in safe_expr):
                raise ValueError("Niedozwolone znaki")

            result = eval(safe_expr)
            if isinstance(result, float) and result.is_integer():
                resultText = f"Wynik: {int(result)}"
            else:
                resultText = f"Wynik: {result}"

            core.callLater(0, ResultDialog, resultText)
            self.Close()

        except Exception:
            core.callLater(0, ResultDialog, "Nieprawidłowe działanie")
            self.Close()

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
    __gestures = {
        "kb:NVDA+control+x": "launchCalculator",
    }

    @script(description="Otwiera kalkulator NVDA")
    def script_launchCalculator(self, gesture):
        core.callLater(0, CalculatorDialog)
